﻿Option Explicit On
Option Strict On

Imports System.Security.Principal
Imports System.Runtime.InteropServices

Public Class Impersonation


    Const LOGON32_LOGON_INTERACTIVE As Integer = 2
    Const LOGON32_PROVIDER_DEFAULT As Integer = 0

    Private Declare Function LogonUserA Lib "advapi32.dll" (ByVal lpszUsername As String, _
                            ByVal lpszDomain As String, _
                            ByVal lpszPassword As String, _
                            ByVal dwLogonType As Integer, _
                            ByVal dwLogonProvider As Integer, _
                            ByRef phToken As IntPtr) As Integer

    Private Declare Auto Function DuplicateToken Lib "advapi32.dll" ( _
                            ByVal ExistingTokenHandle As IntPtr, _
                            ByVal ImpersonationLevel As SECURITY_IMPERSONATION_LEVEL, _
                            ByRef DuplicateTokenHandle As IntPtr) As Integer

    Private Declare Auto Function CloseHandle Lib "kernel32.dll" (ByVal handle As IntPtr) As Long

    Private Enum SECURITY_IMPERSONATION_LEVEL As Integer
        SecurityAnonymous = 0
        SecurityIdentification = 1
        SecurityImpersonation = 2
        SecurityDelegation = 3
    End Enum

    Private m_oImpersonationContext As WindowsImpersonationContext
    Private m_bImpersonationActive As Boolean


    Public ReadOnly Property ImpersonationActive() As Boolean
        '==========================================================================
        ' Name:     ImpersonationActive
        ' Purpose:  Saber si en un momento dado se está efectuando una impersonación del usuario actual
        ' Input:    None
        ' Return:   None
        ' Calls:    None
        '==========================================================================
        Get
            Return m_bImpersonationActive
        End Get
    End Property

    Public Function StartImpersonation(ByVal sUserName As String, ByVal sDomain As String, ByVal sPassword As String) As Boolean
        '==========================================================================
        ' Name:     StartImpersonation
        ' Purpose:  Impersonar un usuario tomando el nombre de otro
        ' Input:    3 string: El nombre del usuario a tomar 'sUserName', El dominio 'sDomain'
        '           El password del usuario a tomar 'sPassword'
        ' Return:   None
        ' Calls:    LogonUserA, DuplicateToken, impersonate de windowsidentity
        '==========================================================================

        Dim bResults As Boolean = False
        Dim sErrorMessage As String
        Dim oWindowsIdentity As WindowsIdentity
        Dim hPrimaryToken As IntPtr = IntPtr.Zero           'a Win32 handle to our authentication token
        Dim hImpersonationToken As IntPtr = IntPtr.Zero     'a Win32 handle to our impersonation token

        If String.IsNullOrEmpty(sUserName) Then
            Throw New ArgumentException("UserName may not be MULL or String.Empty")
        End If

        'Si no se da dominio se toma como local
        If sDomain = String.Empty Then
            sDomain = Environment.MachineName
        End If

        Try
            'Validar el userid, password y dominio dados.
            If LogonUserA(sUserName, sDomain, sPassword, LOGON32_LOGON_INTERACTIVE, LOGON32_PROVIDER_DEFAULT, hPrimaryToken) <> 0 Then

                'Convertir nuestro token a uno que pueda igualrse al TOKEN_IMPERSONATE 
                If DuplicateToken(hPrimaryToken, SECURITY_IMPERSONATION_LEVEL.SecurityImpersonation, hImpersonationToken) <> 0 Then

                    'Crear un nuevo objeto identidad basado en nuestro impersonation token
                    oWindowsIdentity = New WindowsIdentity(hImpersonationToken)

                    'Tomar la nueva identidad
                    m_oImpersonationContext = oWindowsIdentity.Impersonate()
                    If m_oImpersonationContext IsNot Nothing Then
                        m_bImpersonationActive = True
                        bResults = True
                    End If
                Else
                    sErrorMessage = String.Format("DuplicateToken failed (rc={0})", Runtime.InteropServices.Marshal.GetLastWin32Error)
                    Throw New Security.Authentication.AuthenticationException(sErrorMessage)
                End If
            Else
                sErrorMessage = String.Format("LogonUser failed (rc={0})", Runtime.InteropServices.Marshal.GetLastWin32Error)
                Throw New Security.Authentication.AuthenticationException(sErrorMessage)
            End If

        Finally
            If Not hImpersonationToken.Equals(IntPtr.Zero) Then
                CloseHandle(hImpersonationToken)
                hImpersonationToken = IntPtr.Zero
            End If
            If Not hPrimaryToken.Equals(IntPtr.Zero) Then
                CloseHandle(hPrimaryToken)
                hPrimaryToken = IntPtr.Zero
            End If
        End Try

        Return bResults
    End Function

    Public Sub EndImpersonation()
        '==========================================================================
        ' Name:     EndImpersonation
        ' Purpose:  Terminar la impersonación de un usuario 
        ' Input:    None
        ' Return:   None
        ' Calls:    None
        '==========================================================================
        If m_oImpersonationContext IsNot Nothing AndAlso m_bImpersonationActive Then
            'Deshacer la impersonación
            m_oImpersonationContext.Undo()
            m_oImpersonationContext.Dispose()
            m_oImpersonationContext = Nothing
            m_bImpersonationActive = False
        End If
    End Sub

End Class

